
# Note: Use 'setup.py odbctest' to run this; it is not designed to run on its own.

import sys, os, re
import unittest
from decimal import Decimal
import pyodbc

class ODBCTestCase(unittest.TestCase):

    def __init__(self, name, connection_string, map_datatype_to_name):
        unittest.TestCase.__init__(self, name)
        self.connection_string = connection_string
        self.map_datatype_to_name = map_datatype_to_name


    def setUp(self):
        self.cnxn   = pyodbc.connect(self.connection_string)
        self.cursor = self.cnxn.cursor()

        for i in range(3):
            try:
                self.cursor.execute("drop table t%d" % i)
                self.cnxn.commit()
            except:
                pass

        self.cnxn.rollback()

    def tearDown(self):
        try:
            self.cursor.close()
            self.cnxn.close()
        except:
            # If we've already closed the cursor or connection, exceptions are thrown.
            pass

    # Note: Since the Unix ODBC drivers seem to be out of date quite often, use only ODBC 1 SQLGetInfo values.

    def test_getinfo_string(self):
        value = self.cnxn.getinfo(pyodbc.SQL_CATALOG_NAME_SEPARATOR)
        self.assert_(isinstance(value, str))

    def test_getinfo_bool(self):
        value = self.cnxn.getinfo(pyodbc.SQL_ACCESSIBLE_TABLES)
        self.assert_(isinstance(value, bool))

    def test_getinfo_int(self):
        value = self.cnxn.getinfo(pyodbc.SQL_DEFAULT_TXN_ISOLATION)
        self.assert_(isinstance(value, (int, long)))

    def test_getinfo_smallint(self):
        value = self.cnxn.getinfo(pyodbc.SQL_CONCAT_NULL_BEHAVIOR)
        self.assert_(isinstance(value, int))

    def test_guid(self):
        self.cursor.execute("create table t1(g1 uniqueidentifier)")
        self.cursor.execute("insert into t1 values (newid())")
        v = self.cursor.execute("select * from t1").fetchone()[0]
        self.assertEqual(type(v), str)
        self.assertEqual(len(v), 36)

    def test_simple_str(self):
        value = "testing"
        self.cursor.execute("create table t1(s varchar(10))")
        self.cursor.execute("insert into t1 values(?)", "testing")
        v = self.cursor.execute("select * from t1").fetchone()[0]
        self.assertEqual(type(v), str)
        self.assertEqual(len(v), len(value)) # If we alloc'd wrong, the test below might work because of an embedded NULL
        self.assertEqual(v, value)

    def test_fixed_str(self):
        value = "testing"
        self.cursor.execute("create table t1(s char(7))")
        self.cursor.execute("insert into t1 values(?)", "testing")
        v = self.cursor.execute("select * from t1").fetchone()[0]
        self.assertEqual(type(v), str)
        self.assertEqual(len(v), len(value)) # If we alloc'd wrong, the test below might work because of an embedded NULL
        self.assertEqual(v, value)

    def test_fixed_unicode(self):
        value = u"t\xebsting"
        self.cursor.execute("create table t1(s nchar(7))")
        self.cursor.execute("insert into t1 values(?)", u"t\xebsting")
        v = self.cursor.execute("select * from t1").fetchone()[0]
        self.assertEqual(type(v), unicode)
        self.assertEqual(len(v), len(value)) # If we alloc'd wrong, the test below might work because of an embedded NULL
        self.assertEqual(v, value)

    def test_var_unicode(self):
        value = u"test"
        self.cursor.execute("create table t1(s nvarchar(7))")
        self.cursor.execute("insert into t1 values(?)", u"test")
        v = self.cursor.execute("select * from t1").fetchone()[0]
        self.assertEqual(type(v), unicode)
        self.assertEqual(len(v), len(value)) # If we alloc'd wrong, the test below might work because of an embedded NULL
        self.assertEqual(v, value)
        
    def test_long_str(self):
        value = "0123456789" * 300
        self.cursor.execute("create table t1(s varchar(3000))")
        self.cursor.execute("insert into t1 values(?)", value)
        v = self.cursor.execute("select * from t1").fetchone()[0]
        self.assertEqual(type(v), str)
        self.assertEqual(len(v), len(value))
        self.assertEqual(v, value)

    def test_long_unicode(self):
        value =u"t\xebsting" * 300
        self.cursor.execute("create table t1(s nvarchar(3000))")
        self.cursor.execute("insert into t1 values(?)", value)
        v = self.cursor.execute("select * from t1").fetchone()[0]
        self.assertEqual(type(v), unicode)
        self.assertEqual(len(v), len(value))
        self.assertEqual(v, value)

    def test_short_binary(self):
        # pyodbc Uses "normal" binding for buffers 4K and under (actually 4000 bytes?), so we have separate tests for
        # under 4K and over 4K.

        value = buffer("0123456789" * 300)

        # (Some databases uses varbinary w/o sizes.)
        try:
            self.cursor.execute("create table t1(s %s(3000))" % self.map_datatype_to_name[pyodbc.SQL_VARBINARY])
        except:
            self.cursor.execute("create table t1(s %s)" % self.map_datatype_to_name[pyodbc.SQL_VARBINARY])

        self.cursor.execute("insert into t1 values(?)", value)
        v = self.cursor.execute("select * from t1").fetchone()[0]
        self.assertEqual(type(v), buffer)
        self.assertEqual(v, value)

    def test_long_binary(self):
        # pyodbc Uses "normal" binding for buffers 4K and under (actually 4000 bytes?), so we have separate tests for
        # under 4K and over 4K.
        value = buffer("0123456789" * 900)

        # (Some databases uses varbinary w/o sizes.)
        try:
            self.cursor.execute("create table t1(s %s(10000))" % self.map_datatype_to_name[pyodbc.SQL_LONGVARBINARY])
        except:
            self.cursor.execute("create table t1(s %s)" % self.map_datatype_to_name[pyodbc.SQL_LONGVARBINARY])

        self.cursor.execute("insert into t1 values(?)", value)
        v = self.cursor.execute("select * from t1").fetchone()[0]
        self.assertEqual(type(v), buffer)
        self.assertEqual(len(v), len(value))
        self.assertEqual(v, value)


    def test_long_binary_null(self):
        # Bug 1575064:
        self.cursor.execute("create table t1(s %s)" % self.map_datatype_to_name[pyodbc.SQL_VARBINARY])
        self.cursor.execute("insert into t1 values(NULL)")
        v = self.cursor.execute("select * from t1").fetchone()[0]
        self.assertEqual(v, None)


    def test_decimal(self):
        value = Decimal('1234567890987654321')
        self.cursor.execute("create table t1(d numeric(19))")
        self.cursor.execute("insert into t1 values(?)", value)
        v = self.cursor.execute("select * from t1").fetchone()[0]
        self.assertEqual(type(v), Decimal)
        self.assertEqual(v, value)

    def _exec(self):
        self.cursor.execute(self.sql)
        
    def test_close_cnxn(self):
        """Make sure using a Cursor after closing its connection doesn't crash."""

        self.cursor.execute("create table t1(id integer, s varchar(20))")
        self.cursor.execute("insert into t1 values (?,?)", 1, 'test')
        self.cursor.execute("select * from t1")

        self.cnxn.close()
        
        # Now that the connection is closed, we expect an exception.  (If the code attempts to use
        # the HSTMT, we'll get an access violation instead.)
        self.sql = "select * from t1"
        self.assertRaises(pyodbc.ProgrammingError, self._exec)

    def test_empty_string(self):
        self.cursor.execute("create table t1(s varchar(20))")
        self.cursor.execute("insert into t1 values(?)", "")

    def test_empty_unicode(self):
        self.cursor.execute("create table t1(s nvarchar(20))")
        self.cursor.execute("insert into t1 values(?)", u"")

    def test_unicode_query(self):
        self.cursor.execute(u"select 1")

    def test_row_names(self):
        self.cursor.execute("create table t1(s varchar(20))")
        self.cursor.execute("insert into t1 values(?)", "1")
        row = self.cursor.execute("select * from t1").fetchone()
        self.assertEquals(['s'], row.columns)
        
    def test_negative_row_index(self):
        self.cursor.execute("create table t1(s varchar(20))")
        self.cursor.execute("insert into t1 values(?)", "1")
        row = self.cursor.execute("select * from t1").fetchone()
        self.assertEquals(row[0], "1")
        self.assertEquals(row[-1], "1")

    def test_version(self):
        self.assertEquals(3, len(pyodbc.version.split('.'))) # 1.3.1 etc.

    def test_rowcount_delete(self):
        self.assertEquals(self.cursor.rowcount, -1)
        self.cursor.execute("create table t1(i int)")
        count = 4
        for i in range(count):
            self.cursor.execute("insert into t1 values (?)", i)
        self.cursor.execute("delete from t1")
        self.assertEquals(self.cursor.rowcount, count)

    def test_rowcount_nodata(self):
        """
        This represents a different code path than a delete that deleted something.

        The return value is SQL_NO_DATA and code after it was causing an error.  We could use SQL_NO_DATA to step over
        the code that errors out and drop down to the same SQLRowCount code.  On the other hand, we could hardcode a
        zero return value.
        """
        self.cursor.execute("create table t1(i int)")
        # This is a different code path internally.
        self.cursor.execute("delete from t1")
        self.assertEquals(self.cursor.rowcount, 0)

    def test_rowcount_select(self):
        """
        The current version of pyodbc does not attempt to determine the rowcount for select statements.
        """
        self.cursor.execute("create table t1(i int)")
        count = 4
        for i in range(count):
            self.cursor.execute("insert into t1 values (?)", i)
        self.cursor.execute("select * from t1")
        self.assertEquals(self.cursor.rowcount, -1)

    def test_rowcount_reset(self):
        "Ensure rowcount is reset to -1"

        self.cursor.execute("create table t1(i int)")
        count = 4
        for i in range(count):
            self.cursor.execute("insert into t1 values (?)", i)
        self.assertEquals(self.cursor.rowcount, 1)

        self.cursor.execute("create table t2(i int)")
        self.assertEquals(self.cursor.rowcount, -1)

    def test_lower_case(self):
        "Ensure pyodbc.lowercase forces returned column names to lowercase."

        # Has to be set before creating the cursor, so we must recreate self.cursor.

        pyodbc.lowercase = True
        self.cursor = self.cnxn.cursor()

        self.cursor.execute("create table t1(Abc int, dEf int)")
        self.cursor.execute("select * from t1")

        names = [ t[0] for t in self.cursor.description ]
        names.sort()

        self.assertEquals(names, [ "abc", "def" ])

        # Put it back so other tests don't fail.
        pyodbc.lowercase = False
        
    def test_temp_select(self):
        # A project was failing to create temporary tables via select into.
        self.cursor.execute("create table t1(s char(7))")
        self.cursor.execute("insert into t1 values(?)", "testing")
        v = self.cursor.execute("select * from t1").fetchone()[0]
        self.assertEqual(type(v), str)
        self.assertEqual(v, "testing")

        self.cursor.execute("select s into t2 from t1")
        v = self.cursor.execute("select * from t1").fetchone()[0]
        self.assertEqual(type(v), str)
        self.assertEqual(v, "testing")


# Some tests require unusual datatypes (Unicode, GUID).  
MAP_TYPE_TO_TEST = {}

def suite(connection_string, verbose):

    pyodbc.lowercase = True

    cnxn   = pyodbc.connect(connection_string)
    cursor = cnxn.cursor()

    supported = [ (row.type_name, row.data_type) for row in cursor.getTypeInfo() ]

    # A lot of the types returned by SQL Server end with ' identity' and some have '() identity' (e.g. "numeric()
    # identity").  We just want the start.
    r = re.compile('^(\w+)')

    map_datatype_to_name = {}
    for row in cursor.getTypeInfo():
        name = r.search(str(row.type_name)).group(1)
        map_datatype_to_name[row.data_type] = name

    DATATYPES = {
        pyodbc.SQL_BIGINT                    : "SQL_BIGINT",
        pyodbc.SQL_BINARY                    : "SQL_BINARY",
        pyodbc.SQL_BIT                       : "SQL_BIT",
        pyodbc.SQL_CHAR                      : "SQL_CHAR",
        pyodbc.SQL_DECIMAL                   : "SQL_DECIMAL",
        pyodbc.SQL_DOUBLE                    : "SQL_DOUBLE",
        pyodbc.SQL_FLOAT                     : "SQL_FLOAT",
        pyodbc.SQL_GUID                      : "SQL_GUID",
        pyodbc.SQL_INTEGER                   : "SQL_INTEGER",
        pyodbc.SQL_INTEGRITY                 : "SQL_INTEGRITY",
        pyodbc.SQL_INTERVAL_DAY              : "SQL_INTERVAL_DAY",
        pyodbc.SQL_INTERVAL_DAY_TO_HOUR      : "SQL_INTERVAL_DAY_TO_HOUR",
        pyodbc.SQL_INTERVAL_DAY_TO_MINUTE    : "SQL_INTERVAL_DAY_TO_MINUTE",
        pyodbc.SQL_INTERVAL_DAY_TO_SECOND    : "SQL_INTERVAL_DAY_TO_SECOND",
        pyodbc.SQL_INTERVAL_HOUR             : "SQL_INTERVAL_HOUR",
        pyodbc.SQL_INTERVAL_HOUR_TO_MINUTE   : "SQL_INTERVAL_HOUR_TO_MINUTE",
        pyodbc.SQL_INTERVAL_HOUR_TO_SECOND   : "SQL_INTERVAL_HOUR_TO_SECOND",
        pyodbc.SQL_INTERVAL_MINUTE           : "SQL_INTERVAL_MINUTE",
        pyodbc.SQL_INTERVAL_MINUTE_TO_SECOND : "SQL_INTERVAL_MINUTE_TO_SECOND",
        pyodbc.SQL_INTERVAL_MONTH            : "SQL_INTERVAL_MONTH",
        pyodbc.SQL_INTERVAL_SECOND           : "SQL_INTERVAL_SECOND",
        pyodbc.SQL_INTERVAL_YEAR             : "SQL_INTERVAL_YEAR",
        pyodbc.SQL_INTERVAL_YEAR_TO_MONTH    : "SQL_INTERVAL_YEAR_TO_MONTH",
        pyodbc.SQL_LONGVARBINARY             : "SQL_LONGVARBINARY",
        pyodbc.SQL_LONGVARCHAR               : "SQL_LONGVARCHAR",
        pyodbc.SQL_NUMERIC                   : "SQL_NUMERIC",
        pyodbc.SQL_REAL                      : "SQL_REAL",
        pyodbc.SQL_SMALLINT                  : "SQL_SMALLINT",
        pyodbc.SQL_TINYINT                   : "SQL_TINYINT",
        pyodbc.SQL_TYPE_DATE                 : "SQL_TYPE_DATE",
        pyodbc.SQL_TYPE_TIME                 : "SQL_TYPE_TIME",
        pyodbc.SQL_TYPE_TIMESTAMP            : "SQL_TYPE_TIMESTAMP",
        pyodbc.SQL_VARBINARY                 : "SQL_VARBINARY",
        pyodbc.SQL_VARCHAR                   : "SQL_VARCHAR",
        pyodbc.SQL_WCHAR                     : "SQL_WCHAR",
        pyodbc.SQL_WLONGVARCHAR              : "SQL_WLONGVARCHAR",
        pyodbc.SQL_WVARCHAR                  : "SQL_WVARCHAR",
        }


    if verbose:
        print 'Supported types'
        for datatype in sorted(map_datatype_to_name.keys()):
            print ' %5d (%s): %s' % (datatype, DATATYPES.get(datatype, '<?>'), map_datatype_to_name[datatype])

    supported = set(map_datatype_to_name.keys())

    loader = unittest.TestLoader()

    MAP_REQUIRED_TO_NAME = { 'unicode'     : pyodbc.SQL_WVARCHAR,
                             'guid'        : pyodbc.SQL_GUID,
                             'long_binary' : pyodbc.SQL_VARBINARY }

    testnames = []

    for name in loader.getTestCaseNames(ODBCTestCase):

        for fragment, required in MAP_REQUIRED_TO_NAME.items():
            if fragment in name and required not in supported:
                if verbose:
                    print '* %s ignored; no %s support' % (name, DATATYPES.get(required, str(required)))
                break
        else:
            testnames.append(name)

    return unittest.TestSuite([ ODBCTestCase(name, connection_string, map_datatype_to_name) for name in testnames ])
