#!/usr/bin/python

import sys, os
from distutils.core import setup, Command
from distutils.extension import Extension
from distutils.errors import *

# To run the unittests easily, create a setup.cfg file in this directory with entries like the following:

# [apitest]
# connection-string=DRIVER={SQL Server};SERVER=localhost;UID=UID;PWD=PWD;DATABASE=thedb
# 
# [odbctest]
# connection-string=DRIVER={SQL Server};SERVER=localhost;UID=UID;PWD=PWD;DATABASE=thedb

MAJOR = 2
MINOR = 0

def main():

    # I like tying the version number to Subversion, but it requires developers to have svnversion, etc.  This allows
    # us to degrade gracefully and use 0.0.0 (actually, that's the distutils default if you don't supply version).

    version = None
    revision = latest_revision('.')
    if revision:
        version = '%s.%s.%s' % (MAJOR, MINOR, revision)
    else:
        print 'WARNING: Building from a non-Subversion directory or cannot run svnversion.  Using version 0.0.0'

    files = ['pyodbcmodule.cpp', 'cursor.cpp', 'row.cpp', 'connection.cpp', 'buffer.cpp' ]

    if os.name == 'nt':
        files.append('pyodbc.rc')
        libraries=["odbc32"]
    else:
        libraries=["odbc"]

    setup (name = "pyodbc",
           version = version,
           description = "DB API %s.%s Module for ODBC" % (MAJOR, MINOR),

           long_description = ('A Python DB API 2 module for ODBC. This project provides an up-to-date, '
                               'convenient interface to ODBC using native data types like datetime and decimal.'),

           maintainer       = "Michael Kleehammer",
           maintainer_email = "mkleehammer@users.sourceforge.net",

           ext_modules = [ Extension('pyodbc', files,
                                     libraries=libraries,
                                     define_macros=[('PYODBC_REVISION', str(revision))],

                                     ) ],

           classifiers = [ 'Development Status :: 5 - Production/Stable',
                           'Intended Audience :: Developers',
                           'Intended Audience :: System Administrators',
                           'License :: OSI Approved :: MIT License',
                           'Operating System :: Microsoft :: Windows',
                           'Operating System :: POSIX',
                           'Programming Language :: Python',
                           'Topic :: Database',
                           ],

           url = 'http://pyodbc.sourceforge.net',

           cmdclass = dict(tags=TagsCommand,
                           apitest=APITestCommand,
                           odbctest=ODBCTestsCommand,
                           splint=SplintCommand,
                           ),
           )


class TagsCommand(Command):
    description = 'Create a TAGS file for emacs and vim using etags.'

    user_options = [ ]

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        if not self.dry_run:
            os.system('etags *.cpp *.h')


class SplintCommand(Command):
    description = 'Runs splint.exe (an advanced lint) on the code.'

    user_options = [ ]

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        # REVIEW: See if we can get the proper include directory from the compiler object.

        includes = [ 'c:\\Program Files\\Microsoft Visual Studio .NET 2003\\Vc7\\PlatformSDK\\Include',
                     'c:\\dev\\x\\Python-2.5\\Include' ]

        if not self.dry_run:
            options = ' '.join('-I"%s"' % i for i in includes )
            cmd = 'splint %s *.cpp *.h' % options
            print cmd
            os.system(cmd)

    
class APITestCommand(Command):

    description = 'Run the dbapi20 unittests.'

    user_options = [ ('connection-string=', 'c', 'ODBC connection string'),
                     ('verbose=', 'v', 'Run verbose tests'), ]

    def initialize_options(self):
        self.connection_string = None

    def finalize_options(self):
        if not self.connection_string:
            # Find right exception
            raise DistutilsOptionError('Missing requird connection-string option.')

    def run(self):
        if not self.dry_run:
            add_to_path()
            import pyodbc
            print 'pyodbc version:', pyodbc.version

            import dbapi20, unittest, pyodbc

            cs = self.connection_string

            class test_pyodbc(dbapi20.DatabaseAPI20Test):
                driver = pyodbc
                connect_args = [ cs ]
                connect_kw_args = {}

                def test_nextset(self): pass
                def test_setoutputsize(self): pass
                def test_ExceptionsAsConnectionAttributes(self): pass

            suite = unittest.makeSuite(test_pyodbc, 'test')
            testRunner = unittest.TextTestRunner(verbosity=(self.verbose > 1) and 9 or 0)
            result = testRunner.run(suite)


class ODBCTestsCommand(Command):

    description = 'Run the pyodbc unittests.'

    user_options = [ ('connection-string=', 'c', 'ODBC connection string'),
                     ('verbose=', 'v', 'Run verbose tests'),
                     ('test=', 't', 'An individual test to run'), ]

    def initialize_options(self):
        self.connection_string = None
        self.test = 'test'              # Actually the prefix

    def finalize_options(self):
        if not self.connection_string:
            # Find right exception
            raise DistutilsOptionError('Missing requird connection-string option.')

    def run(self):
        if self.dry_run:
            return

        add_to_path()

        import unittest, pyodbc, pyodbctests
        print 'pyodbc version:', pyodbc.version

        cnxn = pyodbc.connect(self.connection_string)
        print 'odbc: %s' % cnxn.getinfo(pyodbc.SQL_ODBC_VER)
        print 'driver: %s %s' % (cnxn.getinfo(pyodbc.SQL_DRIVER_NAME), cnxn.getinfo(pyodbc.SQL_DRIVER_VER))
        print 'driver supports ODBC version %s' % cnxn.getinfo(pyodbc.SQL_DRIVER_ODBC_VER)
        cnxn.close()
        cnxn = None

        # testcase = pyodbctests.ODBCTestCase(self.connection_string)
        # suite = unittest.makeSuite(pyodbctests.ODBCTestCase, self.test)
        suite = pyodbctests.suite(self.connection_string, self.verbose > 1)

        testRunner = unittest.TextTestRunner(verbosity=(self.verbose > 1) and 9 or 0)
        result = testRunner.run(suite)



def getoutput(cmd):
    import os
    pipe = os.popen(cmd, 'r')
    text = pipe.read()
    sts = pipe.close()
    if sts is None: sts = 0
    if text[-1:] == '\n': text = text[:-1]
    return sts, text

def latest_revision(path):

    status, output = getoutput('svnversion -cn %s' % path)

    if output.startswith('exported') or not output.strip():
        return 0

    if ':' in output:
        output = output.split(':', 1)[1]

    if output[-1] == 'M':
        output = output[:-1]

    return int(output)


def add_to_path():
    """
    Prepends the build directory to the path so that newly built pyodbc libraries are used, allowing it to be tested
    without installing it.
    """
    # Put the build directory into the Python path so we pick up the version we just built.
    #
    # To make this cross platform, we'll search the directories until we find the .pyd file.

    import imp

    library_exts  = [ t[0] for t in imp.get_suffixes() if t[-1] == imp.C_EXTENSION ]
    library_names = [ 'pyodbc%s' % ext for ext in library_exts ]

    # Only go into directories that match our version number. 

    dir_suffix = '-%s.%s' % (sys.version_info[0], sys.version_info[1])

    from os.path import join, getsize
    for root, dirs, files in os.walk('build'):
        for d in dirs[:]:
            if not d.endswith(dir_suffix):
                dirs.remove(d)

        for name in library_names:
            if name in files:
                sys.path.insert(0, root)
                return
                
    raise SystemExit('Unable to find the pyodbc library in the build directory.  Have you built it?')


    


if __name__ == '__main__':
    main()
    
